<?php

/**
* 
* @version $Id$
* @copyright 2008
*/
class KeyWord {

	const SPECIAL_PREFIX = "OP ";
	const KEYWORD_PREFIX = "KW ";
	
	private $kwList = array();
	private $specials = array();
	
	static $specialMaxLength = 0;

	/**
	* Vytvoří nový seznam klíčových slov a speciálních znaků
	* 
	* @param array $kw seznam klíčových slov jazyka
	* @param array $spec seznam speciálních znaků
	*/
	public function __construct($kw = false, $spec = false)
	{
		if (is_array($kw)) {
			foreach ($kw as $value => $name) {
				if (is_numeric($value)) {
					$this->addKeyWord($name);
				} else {
					$this->addKeyWord($value, $name);
				}
			}
		} 
		if (is_array($spec)) {
			foreach ($spec as $value => $name) {
				$this->addSpecial($value, $name);
			}
		} 
	} 

	public function addKeyWord($value, $name = false)
	{
		$value = strtolower($value);
		if (!$name) {
			$name = $value;
		}
		if (!is_string($value)) {
			throw new KwException(KwException::WRONG_KW_NAME);
		} 
		if (array_key_exists($value, $this->kwList)) {
			throw new KwException(KwException::DUPLICATE_KW_NAME);
		} 
		$this->kwList[strtolower($value)] = $name;
	} 

	public function addSpecial($value, $name)
	{
		if (!is_string($value)) {
			throw new KwException(KwException::WRONG_KW_NAME);
		} 
		if (array_key_exists($value, $this->specials)) {
			throw new KwException(KwException::DUPLICATE_KW_NAME);
		} 
		if (strlen($value) > self::$specialMaxLength) {
			self::$specialMaxLength = strlen($value);			
		}
		$this->specials[strtolower($value)] = $name;
	} 

	public function isKeyword($value)
	{
		$value = strtolower($value);
		if (!is_string($value)) {
			throw new KException(KwException::WRONG_KW_NAME);
		} 
		if (array_key_exists($value, $this->kwList)) {
			return self::KEYWORD_PREFIX.$this->kwList[$value];
		} else {
			return false;
		} 
	} 

	public function isSpecial($value)
	{
		if (!is_string($value)) {
			throw new KwException(KwException::WRONG_KW_NAME);
		} 
		if (array_key_exists($value, $this->specials)) {
			return self::SPECIAL_PREFIX.$this->specials[$value];
		} else {
			return false;
		} 
	} 
} 

?>