<?php

class Syntan {
	
	protected $table; // rozkladová tabulka
	protected $code; // vstupní kód (výstup z lexikálního analyzátoru)
	protected $startSymbol; // startovní symbol
	protected $token; // aktuální znak jazyka který parsujeme
		
	protected $T; // pole terminálů
	protected $N; // pole neterminálů
	protected $I; // pole výstupních symbolů
	
	protected $logPassed = false; // zda se mají logovat již prošlé stavy 
	protected $passed = array(); // prošlé stavy automatu 
	protected $level = 0; // pomocná proměnná - pamatuje si úroveň zanoření pro prošlé stavy (kvůli výpisu)
	protected $passedTerms = array(); // seznam symbolů která již prošly parserem
	protected $waiting = array(); // pole čekajících terminálů a neterminálů (co parser očekává)
	
	protected $getOutput = false; // zda se mají zachycovat výstupní symboly
	
	protected $output = array(); // pole zachycených výstupních symbolů
	
	protected $position = -1; // startovní pozice ve vstupním kódu
					
	/**
	 * Konstruktor, slouží k inicializaci analyzátoru úvodními hodnotami
	 * 
	 * @access public
	 * @param array $table rozkladová tabulka
	 * @param string $startSymbol Startovní symbol
	 * @param array $terminals terminální abeceda
	 * @param array $nonTerminals neterminální abeceda
	 */
	public function __construct ( & $table = null, $startSymbol = null, $terminals = null, $nonTerminals = null, $infoTags = null ) {
		if ($table != null) {
			$this->table = $table;
		} else {
			$this->table = unserialize(file_get_contents("resultArray.txt"));
		}
		if ($terminals != null) {
			$this->T = $terminals;
		} else {
			$this->T = explode(" ", file_get_contents("terminaly.txt"));
		}
		if ($nonTerminals != null) {
			$this->N = $nonTerminals;
		} else {
			$this->N = explode(" ", file_get_contents("neterminaly.txt"));
		}
		if ($infoTags != null) {
			$this->I = $infoTags;
		} else {
			$this->I = explode(" ", file_get_contents("infoTags.txt"));
		}
		if ($startSymbol != null) {
			$this->startSymbol = $startSymbol;
		} else {
			$this->startSymbol = "file";
		}
		$this->extend($this->startSymbol); 
		$this->savePassed($this->startSymbol);
	}
	
	/**
	* Parsovací funkce - zjistí zda syntaxe zadaného řetězce odpovídá jazyku.
	* 
	* @access public
	* @param string $code kód který se má zkontrolovat
	* @param boolean $logPassed zda se mají logovat stavy automatu
	* @param boolean $output zda se mají logovat a zachycovat výstupní symboly
	*/
	public function parse( $code, $logPassed = false, $output = false) {
		$this->logPassed = $logPassed;
		$this->code = explode(" ", $code);
		$this->getOutput = $output;
		$token = $this->nextToken();
		do {
			if (! $this->isTerminal($token)) {
				throw new SynException(SynException::BAD_SYMBOL, $this->position, $token, $this->waiting[0]);
			}
			if (is_array($this->waiting[0])) {
				// pokud je prvním očekávaným prvkem pole, nacházíme se v neterminálu
				// musíme zjistit které příslušné pravidlo se má použít
				$active = array_shift($this->waiting);
				if (isset($active[$token])) {
					// do pole očekávaných symbolů přidám ty, které odpovídají nalezenému pravidlu
					$this->waiting = array_merge($active[$token], $this->waiting);
				} else {
					// pokud tokenu neodpovídá žádné z pravide, dojde k syntaktické chybě
					throw new SynException(SynException::BAD_SYNTAX, $this->position, $token, $active);
				}
				if (trim($active[$token][0]) !== "") {
					// pokud je očekáván neprázdný token, propadneme se při logování o několik úrovní
					$this->level += sizeof($active[$token]) - 1;
				}
				if ($this->logPassed) {
					$this->savePassed($active[$token], $active["__NON_TERM_NAME__"]);
				}
			} else if ($this->isNonTerminal($this->waiting[0])) {
				// je očekáván neterminál
				if (!isset($this->table[$this->waiting[0]])) {
					// chyba - daný neterminál v rozkladové tabulce neexistuje 
					throw new SynException(SynException::WHAT_THE_HELL, $this->position, $this->waiting[0]);
				}
				// očekávaný neterminál nahradíme polem možných následných stavů
				$this->extend($this->waiting[0]);
			} else if ($this->isTerminal($this->waiting[0])) {
				// je očekáván terminál
				if ($this->waiting[0] === $token) {
					// na vstupu je odpovídající symbol
					$this->passedTerms [] = $token;
					array_shift($this->waiting);
					$token = $this->nextToken();
					$this->level--;
				} else {
					// na vstupu nebyl očekávaný terminál
					throw new SynException(SynException::BAD_SYNTAX, $this->position, $token, $this->waiting[0]);
				}
			} else if ($this->isEpsilon($this->waiting[0])) {
				// prázdné pravidlo - skok o jednu úroveň výš	
				array_shift($this->waiting);
				$this->level--;
			} else if ($this->isOutput($this->waiting[0])) {
				// zachycen výstupní symbol
				if (!$this->getOutput) {
					// nestaráme se o výstupní symboly, nic se neděje
					//nop
				} else if (strpos($this->waiting[0], "_P_") !== false) {
					$this->output [] = array(
						'id' => $this->position - 1, 'name' => $this->waiting[0]
					);
				} else {
					$this->output [] = array (
						'id' => $this->position,
						'name' => $this->waiting[0]
					);
				}
				array_shift($this->waiting);
				$this->level--;
			} else {
				// něco je špatně s očekávaným symbolem 
				throw new SynException(SynException::WHAT_THE_HELL_2, $this->position, $this->waiting[0]);
			}
		} while ($token);
	}
	
	/**
	* Rozšiřuje pole očekávaných symbolů - na jeho první místo dá pole
	* s možnými výstupními stavy.
	* 
	* @params string $nonTerm neterminál - jméno stavu
	* @access private
	*/ 
	private function extend($nonTerm) {
		$newWaiting ["__NON_TERM_NAME__"] = $nonTerm;
		foreach($this->table[$nonTerm] as $symbol => $value){
			$newWaiting [$symbol] = explode(" ", $value);
		}
		$this->waiting[0] = $newWaiting;
	}
	
	public function isTerminal($name){
		return in_array($name, $this->T);
	}
	
	public function isNonTerminal($name){
		return in_array($name, $this->N);
	}
	
	public function isEpsilon($name) {
		return (strlen($name) === 0);
	}
	
	public function isOutput($name) {
		return in_array($name, $this->I);
	}
	
	public function getCode() {
		return $this->code;
	}
	
	public function getPassed() {
		return $this->passed;
	}
	
	public function printPassed() {
		return print_r($this->passed);
	}
	
	public function getPassedTerms() {
		return $this->passedTerms;
	}
	
	public function getOutput() {
		return $this->output;
	}
	
	protected function nextToken() {
		if ( sizeof($this->code) > 0 ) {
			$this->position++;
			return array_shift($this->code);
		} else {
			return false;
		}
	}
	
	/**
	* Loguje prošlé stavy analyzátoru
	*/
	protected function savePassed($pass, $place = "") {
		if(is_array($pass)){
			$pass = implode(" ", $pass);
		}
		if ($place) {
			$pass = $place . " -> " . $pass;
		} 
		if (!trim($pass)) {
			$pass = "&#949;";
		}
		$pass = str_repeat(" ", $this->level ).$pass;
		$this->passed [] = $pass;
	}
}
?>